# import sys
# sys.path.append('/home/jcooloj/src/SharedData/src')
# print(sys.path[0])

import os

from SharedData.Logger import Logger
logger = Logger(__file__,user='master')
from SharedData.Metadata import Metadata
from SharedData.SharedData import SharedData

Logger.log.info('Running on %s!' % (os.name))

# md = Metadata('MASTER/FUT_HEAD')
# md.mergeUpdate(Metadata('MASTER/FUT_TAIL').static)
# md.static

# md = Metadata('VOTER/FUT_HEAD')
# md.mergeUpdate(Metadata('VOTER/FUT_TAIL').static)
# md.static

# md = Metadata('CSI/FUT_HEAD')
# md.mergeUpdate(Metadata('CSI/FUT_TAIL').static)
# md.static

shdata = SharedData('MarketData')
master = shdata['MASTER']['D1']
master['m2m'].dropna(how='all')

import pandas as pd
shdata = SharedData('MarketData')
bvbg = shdata['BVMF/BVBG086']['D1']
df = bvbg[pd.Timestamp('2023-02-14')]
df = df.sort_values('tradqty',ascending=False)

bvbg = shdata['BVMF/BVBG086']['D1']

cme_fut = Metadata('CME/FUT_HEAD')
cme_fut.mergeUpdate(Metadata('CME/FUT_TAIL').static)

fut = Metadata('VOTER/FUT_HEAD')
fut.mergeUpdate(Metadata('VOTER/FUT_TAIL').static)
fut.static.loc['2023-02-14','ES_202303@XCME']

cme_fut.static.loc['2023-02-13','ES_202303@XCME']
cme_fut.static.loc['2023-02-13','MES_202303@XCME']


import pandas as pd
import numpy as np
import io, gzip, hashlib, pickle
import time

from SharedData.Logger import Logger
logger = Logger(__file__,user='master')
from SharedData.SharedData import SharedData
from SharedData.Metadata import Metadata

md = Metadata('SCHEDULES/MASTER_TRADEBOT05')
md.save(True)

md = Metadata('BBG/FIRST_SERIES')
md.static
md.save(True)

keys = Metadata.list('BBG/FUT_CHAIN')
for key in keys:    
    print('saving '+key)
    md = Metadata(key)
    md.save(True)
print('DONE!')
        
# # load data
# shdata = SharedData('MarketData')
# feeder = shdata['BVMF/BVBG086']
# dt = pd.Timestamp('2023-01-11')
# df = feeder['D1'][dt]
# feeder['D1'].tags[dt].Write()

# _df = df.dropna(how='all',axis=0).dropna(how='all',axis=1)
# _df.index.names=['date']
# idx_cols_b = str.encode(','.join(_df.index.names),encoding='UTF-8',errors='ignore')
# _df = _df.reset_index()
# cols_b = str.encode(','.join(_df.columns),encoding='UTF-8',errors='ignore')
# header = np.array([len(idx_cols_b),len(cols_b)]).astype(np.int64)
# # allocate memory
# io_obj = io.BytesIO()
# io_obj.write(header)
# io_obj.write(idx_cols_b)
# io_obj.write(cols_b)

# np.save(io_obj,_df.values,allow_pickle=False)
# #calculate hash
# m = hashlib.md5(io_obj.getbuffer())
# md5hash_b = m.digest()
# io_obj.write(md5hash_b)


# io_obj.seek(0)
# _header = np.frombuffer(io_obj.read(16),dtype=np.int64)
# _idx_cols_b = io_obj.read(_header[0]).decode(encoding='UTF-8',errors='ignore')
# _cols_b = io_obj.read(_header[1]).decode(encoding='UTF-8',errors='ignore')
# _npy = np.load(io_obj)




# feeder.dataset_metadata.save()
# tini = time.time()
# feeder.load()
# print('total time %.2f' % (time.time()-tini))
# feeder['D1'].tags
# feeder.save()
# for md in shdata.metadata:
#     shdata.metadata[md].save()


# df = df.dropna(how='all',axis=0).dropna(how='all',axis=1)
# r, c = df.shape
# idx = (df.index.astype(np.int64))
# idx_b = idx.values.tobytes()
# cols = df.columns.values
# colscsv = ','.join(cols)
# colscsv_b = str.encode(colscsv,encoding='UTF-8',errors='ignore')
# nbidx = len(idx_b)
# nbcols = len(colscsv)
# data = np.ascontiguousarray(df.values.astype(np.float64))
# header = np.array([r,c,nbidx,nbcols,r*c*8]).astype(np.int64)
# #calculate hash
# m = hashlib.md5(idx_b)
# m.update(colscsv_b)
# m.update(data)
# md5hash_b = m.digest()
# # allocate memory
# io_obj = io.BytesIO()        
# io_obj.write(header)
# io_obj.write(idx_b)
# io_obj.write(colscsv_b)
# io_obj.write(data)
# io_obj.write(md5hash_b)

# gzip_io = io.BytesIO()
# gzip_obj = gzip.GzipFile(fileobj=gzip_io, mode='wb', compresslevel=1)
# gzip_obj.write(io_obj.getbuffer())
# gzip_obj.flush()
# len(gzip_io.getbuffer())

# gzip_io.seek(0)
# _gzip_obj = gzip.GzipFile(fileobj=gzip_io,mode='rb')
# _head_io = io.BytesIO()
# _head_io.write(gzip_obj.read())



# tini = time.time()
# # write ziped file in memory
# # create header
# r, c = df.shape
# idx = (df.index.astype(np.int64))
# idx_b = idx.values.tobytes()
# cols = df.columns.values
# colscsv = ','.join(cols)
# colscsv_b = str.encode(colscsv,encoding='UTF-8',errors='ignore')
# nbidx = len(idx_b)
# nbcols = len(colscsv)
# data = df.values.astype(np.float64)
# header = np.array([r,c,nbidx,nbcols,r*c*8]).astype(np.int64)
# #calculate hash
# m = hashlib.md5(idx_b)
# m.update(colscsv_b)
# m.update(data)
# md5hash_b = m.digest()
# # allocate memory
# io_obj = io.BytesIO()
# gzip_obj=gzip.GzipFile(fileobj=io_obj,mode='wb',compresslevel=1)
# gzip_obj.write(header)
# gzip_obj.write(idx_b)
# gzip_obj.write(colscsv_b)
# gzip_obj.write(data)
# gzip_obj.write(md5hash_b)
# gzip_obj.flush()
# print('total time %.2f' % (time.time()-tini))

# # write file to disk
# tini = time.time()
# f = open('m2m.bin.gz','wb')
# f.write(io_obj.getbuffer())
# f.flush()
# f.close()
# print('total time %.2f' % (time.time()-tini))

# # open file in memory
# # io_obj.seek(0)
# io_obj = open('m2m.bin.gz','rb')
# tini = time.time()
# _gzip_obj=gzip.GzipFile(fileobj=io_obj,mode='rb')
# _header = np.frombuffer(_gzip_obj.read(40),dtype=np.int64)
# _idx_b = _gzip_obj.read(int(_header[2]))
# _idx = pd.to_datetime(np.frombuffer(_idx_b,dtype=np.int64))
# _colscsv_b = _gzip_obj.read(int(_header[3]))
# _colscsv = _colscsv_b.decode(encoding='UTF-8',errors='ignore')
# _cols = _colscsv.split(',')
# _data = np.frombuffer(_gzip_obj.read(int(_header[4])),dtype=np.float64).reshape((_header[0],_header[1]))
# _df = pd.DataFrame(_data,index=_idx,columns=_cols)
# #calculate hash
# _m = hashlib.md5(_idx_b)
# _m.update(_colscsv_b)
# _m.update(_data)
# _md5hash_b = _m.digest()
# __md5hash_b = _gzip_obj.read(16)
# assert _md5hash_b==__md5hash_b
# print('total time %.2f' % (time.time()-tini))

# assert (_df.fillna(0)==df.fillna(0)).all().all()




import time
import numpy as np

from SharedData.Logger import Logger
logger = Logger(__file__,user='worker')
from SharedData.SharedDataAWSKinesis import KinesisLogStreamConsumer

SLEEP_TIME = 2

def run():
    Logger.log.info('Starting SharedDataLogsConsumer process')
    consumer = KinesisLogStreamConsumer(user='worker')

    try:
        consumer.readLogs()
        if consumer.connect():
            Logger.log.info('SharedDataLogsConsumer process STARTED!')
    except:
        pass

    while True:
        success = False
        try:
            success = consumer.consume()
        except:
            pass

        if not success:
            Logger.log.info('SharedDataLogsConsumer process error! Restarting...')
            try:
                consumer.readLogs()
                success = consumer.connect()
            except:
                pass
            if success:
                Logger.log.info('SharedDataLogsConsumer process RESTARTED!')
            else:
                Logger.log.info('SharedDataLogsConsumer failed RESTARTING!')
                time.sleep(SLEEP_TIME*5)
        else:
            time.sleep(SLEEP_TIME + SLEEP_TIME*np.random.rand() - SLEEP_TIME/2)

if __name__ == "__main__":
    run()


import numpy as np
import pandas as pd
import time

username = 'master'
from SharedData.Logger import Logger
logger = Logger(__file__,user=username)

from SharedData.Metadata import Metadata
from SharedData.SharedData import SharedData


if __name__ ==  '__main__':
    
    # md = Metadata('CSI/FUT_SERIES')
    # md.static

    # md = Metadata('CSI/FUT_SERIES_SYMBOLS')
    # md.static


    # shdata = SharedData('MarketData')
    # feeder = shdata['VOTER/FUT']

    # tini = time.time()
    # feeder.load()
    # print('total time %f' % (time.time()-tini))

    # tini = time.time()
    # feeder['D1']['m2m']
    # print('total time %f' % (time.time()-tini))

    # tini = time.time()
    # feeder['D1'].tags['m2m'].Write()
    # print('total time %f' % (time.time()-tini))
    
    # md = Metadata('BVMF/BDATUAL/FUT_CHAIN')
    # md.save()

    # feeder['D1'].tags
    # feeder['D1']['m2m'].loc[:feeder['D1']['m2m'].last_valid_index()]

    # tini = time.time()
    # feeder.save(startDate=pd.Timestamp('2000-01-01'))
    # print('total time %f' % (time.time()-tini))

    Logger.log.info('Reading test for %s done!' % (username))


username = 'master'
from SharedData.Logger import Logger
logger = Logger(__file__,user=username)

from SharedData.SharedData import SharedData
import numpy as np
import pandas as pd

shdata = SharedData('MarketData')
feeder = shdata['MASTER']
mktdata = feeder['D1']
df = mktdata['m2m']
df['ES_S0001@XCME'].dropna()

_shdata = SharedData('MarketData',user='ahmaabd')
_mktdata = _shdata['MASTER']['D1']
_df = _mktdata['m2m']
_df['ES_S01@XCME'].dropna()
Logger.log.info('Reading test for %s done!' % (username))


import pandas as pd
import numpy as np
import gzip
import time

from SharedData.Logger import Logger
logger = Logger(__file__)
from SharedData.SharedData import SharedData

shdata = SharedData('MarketData')
feeder = shdata['MASTER']
mktdata = feeder['D1']
df = mktdata['m2m']
r, c = df.shape
idx = (df.index.astype(np.int64)/10**9)
idx_b = idx.values.tobytes()
cols = df.columns.values
colscsv = ','.join(cols)
colscsv_b = str.encode(colscsv,encoding='UTF-8',errors='ignore')
nbidx = len(idx_b)
nbcols = len(colscsv)

tini = time.time()
f=gzip.open('m2m.gz', 'wb',compresslevel=1)
#f=open('m2m.npy', 'wb')
f.write(np.float64(r))
f.write(np.float64(c))
f.write(str.encode(headercsv)+b'\n')
f.write(idx.values.tobytes())
np.save(f,df.values.astype(np.float64))
f.flush()
f.close()
print('total time %.2f' % (time.time()-tini))


tini = time.time()
f=gzip.open('m2m.gz', 'rb')
# f=open('m2m.npy', 'rb')
shape = np.frombuffer(f.read(16))
colscsv = f.readline()[:-1]
cols = str.split(colscsv.decode(),',')
idx = np.frombuffer(f.read(int(8*shape[0])))
index = pd.to_datetime(idx*10**9)
arr = np.load(f)
_df = pd.DataFrame(arr,index=index,columns=cols)
print('total time %.2f' % (time.time()-tini))

assert (_df.fillna(0)==df.fillna(0)).all().all()



import os
import boto3
from botocore.utils import fix_s3_host
from datetime import datetime
import time
import pandas as pd
import numpy as np

from SharedData.Logger import Logger
logger = Logger(__file__,user='master')
from SharedData.MultiProc import io_bound_process
from SharedData.Metadata import Metadata
from SharedData.SharedData import SharedData
from SharedData.SharedDataAWSS3 import S3Download

shdata = SharedData('Signals')
tini = time.time()

shdata['FUTROLL']['D1']['level_delta']
shdata['FUTROLL'].load()
print(time.time()-tini)

# tini = time.time()
# shdata['CSI/FUT'].save()
# print(time.time()-tini)


# # def S3DownloadThread(iteration, args):
# #     return [S3Download(iteration,check_if_exists=False)]
# tini = time.time()
# shm_name = 'master/MarketData/CSI/FUT/D1/m2m'
# bucket_name = os.environ['S3_BUCKET'].replace('s3://','')
# session = boto3.Session()
# s3 = session.resource('s3')
# print(time.time()-tini)
# bucket = s3.Bucket(bucket_name)
# success=True
# dbfolder = os.environ['DATABASE_FOLDER']
# files = np.array([dbfolder+'\\'+obj_s.key.replace('/','\\')\
#     for obj_s in bucket.objects.filter(Prefix=shm_name+'/')])
# tini = time.time()    
# result = io_bound_process(S3DownloadThread,files,None)
# print(time.time()-tini)



# shdata = SharedData('MarketData')
# dt= pd.Timestamp('2022-12-02')
# df = shdata['BVMF/BVBG086']['D1'][dt]

# md = Metadata('test')
# md.static.loc['test','test'] = 'test'
# md.save()
# md = Metadata('test')

# local_path = r'C:\Users\jcooloj\DB\master\Metadata\CSI\FUT_SERIES.pkl'
# bucket_name = os.environ['S3_BUCKET'].replace('s3://','')
# s3_path = str(local_path).replace(os.environ['DATABASE_FOLDER'],'').replace('\\','/')[1:]
# session = boto3.Session()
# s3 = session.resource('s3')
# # obj exists
# bucket = s3.Bucket(bucket_name)
# if len(list(bucket.objects.filter(Prefix=s3_path)))>0:
#     # load obj
#     obj = s3.Object(bucket_name, s3_path)
#     # remote mtime size
#     remote_mtime = obj.last_modified.timestamp()
#     if 'mtime' in obj.metadata:
#         remote_mtime = float(obj.metadata['mtime'])
#     remote_size = obj.content_length
#     # local mtime size
#     local_mtime = datetime.utcfromtimestamp(os.path.getmtime(local_path)).timestamp()
#     local_size = os.path.getsize(local_path)
#     #compare
#     isnewer = remote_mtime>local_mtime
#     ischg = remote_size!=local_size
#     if isnewer or ischg:
#         obj.download_file(local_path)
#         # update modification time
#         remote_mtime_dt = datetime.fromtimestamp(remote_mtime)
#         offset =  remote_mtime_dt - datetime.utcfromtimestamp(remote_mtime)        
#         remote_mtime_local_tz = remote_mtime_dt+offset
#         remote_mtime_local_tz_ts = remote_mtime_local_tz.timestamp()
#         os.utime(local_path, (remote_mtime_local_tz_ts, remote_mtime_local_tz_ts))


# datetime.fromtimestamp(float(obj.metadata['mtime']))
# datetime.utcfromtimestamp(float(obj.metadata['mtime']))

# local_path = 'C:/Users/jcooloj/DB/master/Metadata/test.pkl'
# mtime = int(os.path.getmtime(local_path))
# datetime.utcfromtimestamp(mtime).timestamp()
# mtime_str = str(mtime)


# session = boto3.Session()
# s3 = session.resource('s3',\
#     endpoint_url='http://192.168.0.199:9000',\
#     aws_access_key_id = 'sBj6y99xpZWXHUFZ',\
#     aws_secret_access_key = 'bcKeTVlqacT8t5TiKqWytsOOSvcL2idy')

# s3.meta.client.meta.events.unregister('before-sign.s3', fix_s3_host)
# bucket_name = os.environ['S3_BUCKET'].replace('s3://','')
# bucket = s3.Bucket(bucket_name)

# tini = time.time()
# keys = [obj_s.key for obj_s in bucket.objects.filter(Prefix='master')]
# print('elapsed %.2f' % (time.time()-tini))
# len(keys)


# tini = time.time()
# for obj_s in bucket.objects.filter(Prefix='master'):
#     obj = s3.Object(bucket_name, obj_s.key)
#     #print(obj_s.key,obj_s.size,obj_s.last_modified)
# print('elapsed %.2f' % (time.time()-tini))
    
# local_path = 'C:/Users/jcooloj/DB/master/Metadata/CSI/FIRST_SERIES.pkl'
# mtime = int(os.path.getmtime(local_path))
# mtime_str = str(mtime)

# obj_s = s3.ObjectSummary(bucket_name, 'master/Metadata/CSI/FIRST_SERIES.pkl')
# obj = s3.Object(bucket_name, 'master/Metadata/CSI/FIRST_SERIES.pkl')
# obj_s.last_modified
# datetime.fromtimestamp(float(obj.metadata['mtime']))



username = 'guest'
from SharedData.Logger import Logger
from SharedData.Metadata import Metadata
logger = Logger(__file__,user=username)

from SharedData.Metadata import Metadata
from SharedData.SharedData import SharedData
import numpy as np
import pandas as pd

shdata = SharedData('MarketData')
shdata.dataset
mktdata = shdata['MASTER']['D1']
df = mktdata['m2m']
df['ES_S01@XCME'].dropna()

_shdata = SharedData('MarketData',user=username)
_shdata.dataset = shdata.dataset
_shdata.dataset_metadata.static = shdata.dataset_metadata.static
_shdata.dataset_metadata.save()

md = Metadata('MASTER/FUT')
_md = Metadata('MASTER/FUT',user='guest')
_md.static=md.static
_md.save()

md = Metadata('MASTER/STOCK')
_md = Metadata('MASTER/STOCK',user='guest')
_md.static=md.static
_md.save()

_mktdata = _shdata['MASTER']['D1']
_mktdata['m2m'] = mktdata['m2m']
_df = _mktdata['m2m']
_df['ES_S01@XCME'].dropna()
_mktdata.tags['m2m'].Write()


Logger.log.info('Writing test for %s done!' % (username))


import pandas as pd
import time

from SharedData.Logger import Logger
logger = Logger(__file__)
from SharedData.SharedData import SharedData

shdata = SharedData('MarketData')
feeder = 'CSI'
period = 'D1'

# send shareddataframe
tag = pd.Timestamp('2022-03-04')
data = shdata[feeder][period]
df = data[tag]
data.tags[tag].Broadcast(
    idx=['01_202203@XKFE','01_202206@XKFE'],
    col=['open','high'])

# send timeseries
tag = 'close'
data = shdata[feeder][period]
df = data[tag]
idx = df.index[-2:].values
col=['01_S01@XKFE','01_S02@XKFE']
df.loc[idx,col]
data.tags[tag].Broadcast(idx = idx,col=col)


import pandas as pd
import time

from SharedData.Logger import Logger
logger = Logger(__file__)
from SharedData.SharedData import SharedData

shdata = SharedData('MarketData')
wd = shdata['MASTER/watchdog']['D1']
today = pd.Timestamp(pd.Timestamp.now().date())
dfwatchdog = wd[today]
if dfwatchdog.empty:
    df = pd.DataFrame(
        0,
        index = [shdata.database],
        columns = ['watchdog']
    )
    wd[today] = df




import pandas as pd


from SharedData.Logger import Logger
logger = Logger(__file__)

from SharedData.Metadata import Metadata
from SharedData.SharedData import SharedData

futchain = Metadata('BBG/FUT_CHAIN')
futseries = Metadata('BBG/FUT_SERIES')
lvidx = futseries.symbols.last_valid_index()
activefuts = futseries.symbols.loc[lvidx].dropna()
activefuts = activefuts[[s in futchain.static.index for s in activefuts.values]]
activefuts = pd.DataFrame(activefuts)
activefuts.columns=['symbol']
activefuts['bloombergsymbol'] = futchain.static.loc[activefuts['symbol']]['bloombergsymbol'].values
tickers = activefuts['bloombergsymbol'].values
# tickers = ['XBTUSD CURNCY']
columns = ['bid','ask','last_price','volume']
df = pd.DataFrame(index=tickers,columns=columns,dtype=float)

shdata = SharedData('MarketData','w')
feeder = shdata['BBG/RT']['D1']
dt = pd.Timestamp(pd.Timestamp.now().date())
feeder[dt] = df
feeder.tags[dt].Write()


import time

from SharedData.Logger import Logger
from SharedData.SharedDataAWSKinesis import KinesisStreamConsumer

streamname = 'deepportfolio-real-time-marketdata'

logger = Logger(__file__)
Logger.log.info('Starting Kinesis Stream Consumer Loop')
consumer = KinesisStreamConsumer(streamname)

while True:
    consumer.consume()
    for record in consumer.stream_buffer:
        print(record)
    consumer.stream_buffer = []
    time.sleep(1)

    


import numpy as np
import pandas as pd
import time

username = 'master'
from SharedData.Logger import Logger
logger = Logger(__file__,user=username)

from SharedData.SharedData import SharedData


if __name__ ==  '__main__':
    
    shdata = SharedData('MarketData')
    feeder = shdata['BVMF/FUT']
    
    tini = time.time()
    feeder.load()
    print('total time %f' % (time.time()-tini))


    # tini = time.time()
    # for tag in feeder.dataset['tag']:
    #     feeder['D1'][tag]
    # print('total time %f' % (time.time()-tini))

    # tini = time.time()
    # feeder = shdata['TEST']
    # feeder.default_collections='MASTER/FUT,MASTER/STOCK'
    # feeder.load_dataset(tags=['test1','test2'])
    # print('total time %f' % (time.time()-tini))


    mktdata=feeder['D1']
    mktdata.tags
    mktdata['ret'].loc[:mktdata['ret'].last_valid_index()]
    mktdata['m2m'].loc[:mktdata['m2m'].last_valid_index()]
    
    Logger.log.info('Reading test for %s done!' % (username))


import matplotlib.pyplot as plt
plt.style.use('default')
import pandas as pd

from SharedData.Logger import Logger
logger = Logger(__file__)

import numpy as np
message = 'Logging test %2.10f' % (np.random.rand())
Logger.log.info(message)
dflogs = logger.readLogs()
_dflogs = dflogs[dflogs['name']==logger.source]
if (_dflogs.iloc[-1]['message'] == message):
    Logger.log.info('logging works!')
else:
    raise Exception('SharedData logging error!')


import pandas as pd
import numpy as np
from multiprocessing import shared_memory
import time
from numba import njit,prange

from SharedData.Logger import Logger
logger = Logger(__file__,user='master')
from SharedData.Metadata import Metadata


md = Metadata('MASTER/FUT_HEAD')
md.mergeUpdate(Metadata('MASTER/FUT_TAIL').static)

df = md.static.reset_index()
dtypes = df.dtypes.reset_index()
dtypes.columns = ['tag','dtype']

tags_obj =  dtypes['tag'][dtypes['dtype']=='object'].values
tag = tags_obj[0]
for tag in tags_obj:
    df[tag] = df[tag].astype('|S')

dtypes = df.dtypes.reset_index()
dtypes.columns = ['tag','dtype']
# dtypes['dtype'] = dtypes['dtype'].astype(str)
# dtypes = dtypes.sort_values(['dtype','tag'])
# df = df[dtypes['tag']].copy()

_df = df.to_records(index=False)
_df[0]
_df.dtype


shm = shared_memory.SharedMemory(name = 'test',create=True, size=_df.size * _df.itemsize)
shmarr = np.ndarray(_df.shape,dtype=_df.dtype, buffer=shm.buf)
shmarr[:] = _df[:]
shmarr[0]


_shm = shared_memory.SharedMemory(name = 'test',create=False)
_shmarr = np.ndarray(_df.shape,dtype=_df.dtype, buffer=_shm.buf)
_shmarr[0]['m2m']=1

shmarr[0]['m2m']


tini = time.time()
for i in range(shmarr.shape[0]):
    if shmarr[i]['serie']=='DOL_S0001@BVMF':
        pass
print(time.time()-tini)

__df = pd.DataFrame(_shmarr, copy=False)
__df.loc[0,'m2m']=2
df.dtypes

@njit(parallel=True)
def test(shmarr):    
    for i in prange(shmarr.shape[0]):
        if shmarr[i]['serie']=='DOL_S0001@BVMF':
            shmarr[i]['m2m_notional']=shmarr[i]['m2m']*shmarr[i]['multiplier']

@njit(parallel=True)
def testloop(shmarr):
    for j in range(1000000):
        test(shmarr)

tini = time.time()
testloop(shmarr)
print(time.time()-tini)





import numpy as np
import pandas as pd
import time

username = 'master'
from SharedData.Logger import Logger
logger = Logger(__file__,user=username)

from SharedData.SharedData import SharedData


if __name__ ==  '__main__':
    
    shdata = SharedData('MarketData')
    feeder = shdata['MASTER']
    
    tini = time.time()
    feeder.load()
    
    print('total time %f' % (time.time()-tini))
    
    feeder.dataset_scan()

    stats_tags = ['create_map','init_time',\
        'last_valid_index','last_update','first_update',\
        'data_size','memory_size',\
        'download_time','download_speed',\
        'index_count','columns_count',\
        'notnull_sum','notnull_index','notnull_columns',\
        'density_ratio','density_ratio_index','density_ratio_columns']
    ds = feeder.dataset_metadata.static.set_index('tag')
    idx = ds['density_ratio']>0.0
    ds.loc[idx,stats_tags]
    
    # tini = time.time()
    # feeder.load()
    # print('total time %f' % (time.time()-tini))

    # tini = time.time()
    # for tag in feeder.dataset['tag']:
    #     feeder['D1'][tag]
    # print('total time %f' % (time.time()-tini))

    # tini = time.time()
    # feeder = shdata['TEST']
    # feeder.default_collections='MASTER/FUT,MASTER/STOCK'
    # feeder.load_dataset(tags=['test1','test2'])
    # print('total time %f' % (time.time()-tini))


    mktdata=feeder['D1']
    mktdata.tags
    mktdata['ret'].loc[:mktdata['ret'].last_valid_index()]
    mktdata['m2m'].loc[:mktdata['m2m'].last_valid_index()]
    
    Logger.log.info('Reading test for %s done!' % (username))